@@ -27,9 +27,8 @@
 #include <unistd.h>
 #include <errno.h>
 
-
 /* Some of the following code to define major/minor taken from code by
- * Jörg Schilling's star archiver.
+ * Jrg Schilling's star archiver.
  */
 #if !defined(major) && (defined(sgi) || defined(__sgi) || defined(__SVR4)) && !defined(__CYGWIN32__)
 #include <sys/mkdev.h>
@@ -58,30 +57,20 @@
     #define PY_LONG_LONG LONG_LONG 
 #endif
 
-/* The following section is by Jeffrey A. Marshall and compensates for
- * a bug in Mac OS X's S_ISFIFO and S_ISSOCK macros.
- */
-#ifdef __APPLE__
-/* S_ISFIFO/S_ISSOCK macros from <sys/stat.h> on mac osx are bogus */
-#undef S_ISSOCK               /* their definition of a socket includes fifos */
-#undef S_ISFIFO               /* their definition of a fifo includes sockets */
-#define S_ISSOCK(mode)        (((mode) & S_IFMT) == S_IFSOCK)
-#define S_ISFIFO(mode)        (((mode) & S_IFMT) == S_IFIFO)
-#endif
-
 static PyObject *UnknownFileTypeError;
 static PyObject *c_make_file_dict(PyObject *self, PyObject *args);
 static PyObject *long2str(PyObject *self, PyObject *args);
 static PyObject *str2long(PyObject *self, PyObject *args);
 static PyObject *my_sync(PyObject *self, PyObject *args);
 
+
 /* Turn a stat structure into a python dictionary.  The preprocessor
    stuff taken from Python's posixmodule.c */
 static PyObject *c_make_file_dict(self, args)
 	 PyObject *self;
 	 PyObject *args;
 {
-  PyObject *size, *inode, *mtime, *atime, *devloc, *return_val;
+  PyObject *size, *inode, *mtime, *atime, *ctime, *devloc, *return_val;
   char *filename, filetype[5];
   STRUCT_STAT sbuf;
   long int mode, perms;
@@ -118,9 +107,11 @@ static PyObject *c_make_file_dict(self, args)
 #if SIZEOF_TIME_T > SIZEOF_LONG
   mtime = PyLong_FromLongLong((PY_LONG_LONG)sbuf.st_mtime);
   atime = PyLong_FromLongLong((PY_LONG_LONG)sbuf.st_atime);
+  ctime = PyLong_FromLongLong((PY_LONG_LONG)sbuf.st_ctime);
 #else
   mtime = PyInt_FromLong((long)sbuf.st_mtime);
   atime = PyInt_FromLong((long)sbuf.st_atime);
+  ctime = PyInt_FromLong((long)sbuf.st_ctime);
 #endif
 
   /* Build return dictionary from stat struct */
@@ -130,7 +121,7 @@ static PyObject *c_make_file_dict(self, args)
 	else if S_ISDIR(mode) strcpy(filetype, "dir");
 	else if S_ISSOCK(mode) strcpy(filetype, "sock");
 	else strcpy(filetype, "fifo");
-	return_val =  Py_BuildValue("{s:s,s:O,s:l,s:l,s:l,s:O,s:O,s:l,s:O,s:O}",
+	return_val =  Py_BuildValue("{s:s,s:O,s:l,s:l,s:l,s:O,s:O,s:l,s:O,s:O,s:O}",
 								"type", filetype,
 								"size", size,
 								"perms", perms,
@@ -140,7 +131,8 @@ static PyObject *c_make_file_dict(self, args)
 								"devloc", devloc,
 								"nlink", (long)sbuf.st_nlink,
 								"mtime", mtime,
-								"atime", atime);
+								"atime", atime,
+								"ctime", ctime);
   } else if S_ISLNK(mode) {
 	/* Symbolic links */
 	char linkname[1024];
@@ -169,7 +161,7 @@ static PyObject *c_make_file_dict(self, args)
 	PyObject *major_num = PyLong_FromLongLong(major(devnums));
 #else
 	long int devnums = (long)sbuf.st_dev;
-	PyObject *major_num = PyInt_FromLong(devnums >> 8);
+	PyObject *major_num = PyInt_FromLong(major(devnums));
 #endif
 	int minor_num = (int)(minor(devnums));
 	if S_ISCHR(mode) strcpy(devtype, "c");
@@ -196,6 +188,7 @@ static PyObject *c_make_file_dict(self, args)
   Py_DECREF(devloc);
   Py_DECREF(mtime);
   Py_DECREF(atime);
+  Py_DECREF(ctime);
   return return_val;
 }
 
