@@ -23,7 +23,7 @@ import re, os
 
 
 # The current version of rdiff-backup
-version = "0.12.8"
+version = "0.13.0"
 
 # If this is set, use this value in seconds as the current time
 # instead of reading it from the clock.
@@ -53,6 +53,9 @@ process_gid = os.getgid()
 # If true, when copying attributes, also change target's uid/gid
 change_ownership = None
 
+# If true, when copying attributes, also change target's permission.
+change_permissions = 1
+
 # If true, change the permissions of unwriteable mirror files
 # (such as directories) so that they can be written, and then
 # change them back.  This defaults to 1 just in case the process
@@ -63,6 +66,29 @@ change_mirror_perms = (process_uid != 0)
 # If true, try to reset the atimes of the source partition.
 preserve_atime = None
 
+# If true, save the extended attributes when backing up.
+read_eas = None
+
+# If true, preserve the extended attributes on the mirror directory
+# when backing up, or write them to the restore directory.  This
+# requires read_eas.
+write_eas = None
+
+# If true, save access control lists when backup up.
+read_acls = None
+
+# If true, write access control list information to the destination
+# when backing up or restoring.  Requires read_acls.
+write_acls = None
+
+# If true, look for and save resource fork information when backing
+# up.
+read_resource_forks = None
+
+# If true, write resource fork information to destination when backing
+# up or restoring.  Requires read_resource_forks.
+write_resource_forks = None
+
 # This will be set as soon as the LocalConnection class loads
 local_connection = None
 
@@ -107,12 +133,10 @@ changed_settings = []
 # The RPath or QuotedRPath of the rdiff-backup-data directory.
 rbdir = None
 
-# quoting_enabled is true if we should quote certain characters in
-# filenames on the source side (see FilenameMapping for more
-# info).  chars_to_quote is a string whose characters should be
-# quoted, and quoting_char is the character to quote with.
-quoting_enabled = None
-chars_to_quote = "A-Z:"
+# chars_to_quote is a string whose characters should be quoted.  It
+# should be true if certain characters in filenames on the source side
+# should be escaped (see FilenameMapping for more info).
+chars_to_quote = None
 quoting_char = ';'
 
 # If true, emit output intended to be easily readable by a
@@ -135,7 +159,7 @@ compression = 1
 # to .snapshots and .diffs).  The second below will be the
 # compiled version of the first.
 no_compression_regexp_string = "(?i).*\\.(gz|z|bz|bz2|tgz|zip|rpm|deb|" \
-						"jpg|gif|png|jp2|mp3|ogg|avi|wmv|mpeg|mpg|rm|mov|flac|shn)$"
+						"jpg|gif|png|jp2|mp3|ogg|avi|wmv|mpeg|mpg|rm|mov)$"
 no_compression_regexp = None
 
 # If true, filelists and directory statistics will be split on
