@@ -1,4 +1,4 @@
-# Copyright 2002 Ben Escoto
+# Copyright 2002, 2003 Ben Escoto
 #
 # This file is part of rdiff-backup.
 #
@@ -20,11 +20,11 @@
 """Start (and end) here - read arguments, set global settings, etc."""
 
 from __future__ import generators
-import getopt, sys, re, os, cStringIO
+import getopt, sys, re, os
 from log import Log, LoggerError, ErrorLog
 import Globals, Time, SetConnections, selection, robust, rpath, \
 	   manage, backup, connection, restore, FilenameMapping, \
-	   Security, Hardlink, regress, C, statistics
+	   Security, Hardlink, regress, C, fs_abilities
 
 
 action = None
@@ -32,6 +32,9 @@ remote_cmd, remote_schema = None, None
 force = None
 select_opts = []
 select_files = []
+# These are global because they are set while we are trying to figure
+# whether to restore or to backup
+restore_root, restore_index, restore_root_set = None, None, 0
 
 def parse_cmdlineoptions(arglist):
 	"""Parse argument list and set global preferences"""
@@ -42,30 +45,26 @@ def parse_cmdlineoptions(arglist):
 		try: return open(filename, "r")
 		except IOError: Log.FatalError("Error opening file %s" % filename)
 
-	def normalize_path(path):
-		"""Used below to normalize the security paths before setting"""
-		return rpath.RPath(Globals.local_connection, path).normalize().path
-
 	try: optlist, args = getopt.getopt(arglist, "blr:sv:V",
-		 ["backup-mode", "calculate-average", "chars-to-quote=",
-		  "check-destination-dir", "current-time=", "exclude=",
-		  "exclude-device-files", "exclude-filelist=",
-		  "exclude-filelist-stdin", "exclude-globbing-filelist=",
-		  "exclude-mirror=", "exclude-other-filesystems",
-		  "exclude-regexp=", "exclude-special-files", "force",
-		  "include=", "include-filelist=", "include-filelist-stdin",
+		 ["backup-mode", "calculate-average", "check-destination-dir",
+		  "current-time=", "exclude=", "exclude-device-files",
+		  "exclude-filelist=", "exclude-filelist-stdin",
+		  "exclude-globbing-filelist=", "exclude-mirror=",
+		  "exclude-other-filesystems", "exclude-regexp=",
+		  "exclude-special-files", "force", "include=",
+		  "include-filelist=", "include-filelist-stdin",
 		  "include-globbing-filelist=", "include-regexp=",
 		  "list-at-time=", "list-changed-since=", "list-increments",
-		  "list-increment-sizes", "no-change-dir-inc-perms",
-		  "no-compare-inode", "no-compression",
+		  "list-increment-sizes", "no-compare-inode",
+		  "no-change-dir-inc-perms", "no-compression",
 		  "no-compression-regexp=", "no-file-statistics",
-		  "no-hard-links", "null-separator", "parsable-output",
-		  "print-statistics", "quoting-char=", "remote-cmd=",
-		  "remote-schema=", "remove-older-than=", "restore-as-of=",
-		  "restrict=", "restrict-read-only=", "restrict-update-only=",
-		  "server", "ssh-no-compression", "terminal-verbosity=",
-		  "test-server", "verbosity=", "version", "windows-mode",
-		  "windows-restore"])
+		  "no-hard-links", "null-separator",
+		  "override-chars-to-quote=", "parsable-output",
+		  "print-statistics", "remote-cmd=", "remote-schema=",
+		  "remove-older-than=", "restore-as-of=", "restrict=",
+		  "restrict-read-only=", "restrict-update-only=", "server",
+		  "ssh-no-compression", "terminal-verbosity=", "test-server",
+		  "verbosity=", "version"])
 	except getopt.error, e:
 		commandline_error("Bad commandline options: %s" % str(e))
 
@@ -73,9 +72,6 @@ def parse_cmdlineoptions(arglist):
 		if opt == "-b" or opt == "--backup-mode": action = "backup"
 		elif opt == "--calculate-average": action = "calculate-average"
 		elif opt == "--check-destination-dir": action = "check-destination-dir"
-		elif opt == "--chars-to-quote":
-			Globals.set('chars_to_quote', arg)
-			Globals.set('quoting_enabled', 1)
 		elif opt == "--current-time":
 			Globals.set_integer('current_time', arg)
 		elif opt == "--exclude": select_opts.append((opt, arg))
@@ -111,8 +107,8 @@ def parse_cmdlineoptions(arglist):
 		elif opt == "-l" or opt == "--list-increments":
 			action = "list-increments"
 		elif opt == '--list-increment-sizes': action = 'list-increment-sizes'
-		elif opt == "--no-change-dir-inc-perms":
-			Globals.set("change_dir_inc_perms", 0)
+		elif opt == '--no-change-dir-inc-perms':
+			Globals.set('change_dir_inc_perms', 0)
 		elif opt == "--no-compare-inode": Globals.set("compare_inode", 0)
 		elif opt == "--no-compression": Globals.set("compression", None)
 		elif opt == "--no-compression-regexp":
@@ -120,11 +116,10 @@ def parse_cmdlineoptions(arglist):
 		elif opt == "--no-file-statistics": Globals.set('file_statistics', 0)
 		elif opt == "--no-hard-links": Globals.set('preserve_hardlinks', 0)
 		elif opt == "--null-separator": Globals.set("null_separator", 1)
+		elif opt == "--override-chars-to-quote":
+			Globals.set('chars_to_quote', arg)
 		elif opt == "--parsable-output": Globals.set('parsable_output', 1)
 		elif opt == "--print-statistics": Globals.set('print_statistics', 1)
-		elif opt == "--quoting-char":
-			Globals.set('quoting_char', arg)
-			Globals.set('quoting_enabled', 1)
 		elif opt == "-r" or opt == "--restore-as-of":
 			restore_timestr, action = arg, "restore-as-of"
 		elif opt == "--remote-cmd": remote_cmd = arg
@@ -132,13 +127,13 @@ def parse_cmdlineoptions(arglist):
 		elif opt == "--remove-older-than":
 			remove_older_than_string = arg
 			action = "remove-older-than"
-		elif opt == "--restrict": Globals.restrict_path = normalize_path(arg)
+		elif opt == "--restrict": Globals.restrict_path = arg
 		elif opt == "--restrict-read-only":
 			Globals.security_level = "read-only"
-			Globals.restrict_path = normalize_path(arg)
+			Globals.restrict_path = arg
 		elif opt == "--restrict-update-only":
 			Globals.security_level = "update-only"
-			Globals.restrict_path = normalize_path(arg)
+			Globals.restrict_path = arg
 		elif opt == "-s" or opt == "--server":
 			action = "server"
 			Globals.server = 1
@@ -150,56 +145,30 @@ def parse_cmdlineoptions(arglist):
 			print "rdiff-backup " + Globals.version
 			sys.exit(0)
 		elif opt == "-v" or opt == "--verbosity": Log.setverbosity(arg)
-		elif opt == "--windows-mode":
-			Globals.set('chars_to_quote', "^a-z0-9._ -")
-			Globals.set('quoting_enabled', 1)
-			Globals.set('preserve_hardlinks', 0)
-			Globals.set('change_ownership', 0)
-			Globals.set('change_permissions', 0)
-			Globals.set('fsync_directories', 0)
-		elif opt == '--windows-restore':
-			Globals.set('chars_to_quote', "^a-z0-9._ -")
-			Globals.set('quoting_enabled', 1)
 		else: Log.FatalError("Unknown option %s" % opt)
 
-def isincfilename(path):
-	"""Return true if path is of a (possibly quoted) increment file"""
-	rp = rpath.RPath(Globals.local_connection, path)
-	if Globals.quoting_enabled:
-		if not FilenameMapping.quoting_char:
-			FilenameMapping.set_init_quote_vals()
-		rp = FilenameMapping.get_quotedrpath(rp, separate_basename = 1)
-	result = rp.isincfile()
-	return result
-
-def set_action():
-	"""Check arguments and try to set action"""
+def check_action():
+	"""Check to make sure action is compatible with args"""
 	global action
+	arg_action_dict = {0: ['server'],
+					   1: ['list-increments', 'list-increment-sizes',
+						   'remove-older-than', 'list-at-time',
+						   'list-changed-since', 'check-destination-dir'],
+					   2: ['backup', 'restore', 'restore-as-of']}
 	l = len(args)
-	if not action:
+	if not action: assert l == 2, args # cannot tell backup or restore yet
+	elif action == 'calculate-average':
 		if l == 0: commandline_error("No arguments given")
-		elif l == 1: action = "restore"
-		elif l == 2:
-			if isincfilename(args[0]): action = "restore"
-			else: action = "backup"
-		else: commandline_error("Too many arguments given")
+	elif l > 2 or action not in arg_action_dict[l]:
+		commandline_error("Wrong number of arguments given.  See man page.")
 
-	if l == 0 and action != "server":
-		commandline_error("No arguments given")
-	if l > 0 and action == "server":
-		commandline_error("Too many arguments given")
-	if l < 2 and (action == "backup" or action == "restore-as-of"):
-		commandline_error("Two arguments are required (source, destination).")
-	if l == 2 and (action == "list-increments" or
-				   action == "list-increment-sizes" or
-				   action == "remove-older-than" or
-				   action == "list-at-time" or
-				   action == "list-changed-since" or
-				   action == "check-destination-dir"):
-		commandline_error("Only use one argument, "
-						  "the root of the backup directory")
-	if l > 2 and action != "calculate-average":
-		commandline_error("Too many arguments given")
+def final_set_action(rps):
+	"""If no action set, decide between backup and restore at this point"""
+	global action
+	if action: return
+	assert len(rps) == 2, rps
+	if restore_set_root(rps[0]): action = "restore"
+	else: action = "backup"
 
 def commandline_error(message):
 	sys.stderr.write("Error: %s\n" % message)
@@ -210,7 +179,6 @@ def misc_setup(rps):
 	"""Set default change ownership flag, umask, relay regexps"""
 	os.umask(077)
 	Time.setcurtime(Globals.current_time)
-	FilenameMapping.set_init_quote_vals()
 	SetConnections.UpdateGlobal("client_conn", Globals.local_connection)
 	Globals.postset_regexp('no_compression_regexp',
 						   Globals.no_compression_regexp_string)
@@ -225,12 +193,12 @@ def take_action(rps):
 		sys.exit(0)
 	elif action == "backup": Backup(rps[0], rps[1])
 	elif action == "restore": Restore(*rps)
-	elif action == "restore-as-of": RestoreAsOf(rps[0], rps[1])
+	elif action == "restore-as-of": Restore(rps[0], rps[1], 1)
 	elif action == "test-server": SetConnections.TestConnections()
 	elif action == "list-at-time": ListAtTime(rps[0])
 	elif action == "list-changed-since": ListChangedSince(rps[0])
 	elif action == "list-increments": ListIncrements(rps[0])
-	elif action == "list-increment-sizes": ListIncrementSizes(rps[0])
+	elif action == 'list-increment-sizes': ListIncrementSizes(rps[0])
 	elif action == "remove-older-than": RemoveOlderThan(rps[0])
 	elif action == "calculate-average": CalculateAverage(rps)
 	elif action == "check-destination-dir": CheckDest(rps[0])
@@ -246,10 +214,11 @@ def cleanup():
 def Main(arglist):
 	"""Start everything up!"""
 	parse_cmdlineoptions(arglist)
-	set_action()
+	check_action()
 	cmdpairs = SetConnections.get_cmd_pairs(args, remote_schema, remote_cmd)
-	Security.initialize(action, cmdpairs)
+	Security.initialize(action or "mirror", cmdpairs)
 	rps = map(SetConnections.cmdpair2rp, cmdpairs)
+	final_set_action(rps)
 	misc_setup(rps)
 	take_action(rps)
 	cleanup()
@@ -257,14 +226,18 @@ def Main(arglist):
 
 def Backup(rpin, rpout):
 	"""Backup, possibly incrementally, src_path to dest_path."""
-	if Globals.quoting_enabled:
-		rpout = FilenameMapping.get_quotedrpath(rpout)
 	SetConnections.BackupInitConnections(rpin.conn, rpout.conn)
+	backup_check_dirs(rpin, rpout)
+	backup_set_fs_globals(rpin, rpout)
+	if Globals.chars_to_quote:
+		rpout = FilenameMapping.get_quotedrpath(rpout)
+		SetConnections.UpdateGlobal(
+			'rbdir', FilenameMapping.get_quotedrpath(Globals.rbdir))
+	backup_set_rbdir(rpin, rpout)
 	backup_set_select(rpin)
-	backup_init_dirs(rpin, rpout)
 	if prevtime:
-		Time.setprevtime(prevtime)
 		rpout.conn.Main.backup_touch_curmirror_local(rpin, rpout)
+		Time.setprevtime(prevtime)
 		backup.Mirror_and_increment(rpin, rpout, incdir)
 		rpout.conn.Main.backup_remove_curmirror_local()
 	else:
@@ -276,31 +249,39 @@ def backup_set_select(rpin):
 	rpin.conn.backup.SourceStruct.set_source_select(rpin, select_opts,
 													*select_files)
 
-def backup_init_dirs(rpin, rpout):
-	"""Make sure rpin and rpout are valid, init data dir and logging"""
-	global datadir, incdir, prevtime
+def backup_check_dirs(rpin, rpout):
+	"""Make sure in and out dirs exist and are directories"""
 	if rpout.lstat() and not rpout.isdir():
 		if not force: Log.FatalError("Destination %s exists and is not a "
 									 "directory" % rpout.path)
 		else:
 			Log("Deleting %s" % rpout.path, 3)
 			rpout.delete()
+	if not rpout.lstat():
+		try: rpout.mkdir()
+		except os.error:
+			Log.FatalError("Unable to create directory %s" % rpout.path)
 
 	if not rpin.lstat():
 		Log.FatalError("Source directory %s does not exist" % rpin.path)
 	elif not rpin.isdir():
 		Log.FatalError("Source %s is not a directory" % rpin.path)
+	backup_warn_if_infinite_regress(rpin, rpout)
+	Globals.rbdir = rpout.append_path("rdiff-backup-data")
 
-	datadir = rpout.append_path("rdiff-backup-data")
-	SetConnections.UpdateGlobal('rbdir', datadir)
+def backup_set_rbdir(rpin, rpout):
+	"""Initialize data dir and logging"""
+	global incdir, prevtime
+	SetConnections.UpdateGlobal('rbdir', Globals.rbdir)
 	checkdest_if_necessary(rpout)
-	incdir = datadir.append_path("increments")
+	incdir = Globals.rbdir.append_path("increments")
 	prevtime = backup_get_mirrortime()
 
-	if rpout.lstat():
-		if rpout.isdir() and not rpout.listdir(): # rpout is empty dir
+	assert rpout.lstat(), (rpout.path, rpout.lstat())
+	if rpout.isdir() and not rpout.listdir(): # rpout is empty dir
+		if Globals.change_permissions:
 			rpout.chmod(0700) # just make sure permissions aren't too lax
-		elif not datadir.lstat() and not force: Log.FatalError(
+	elif not Globals.rbdir.lstat() and not force: Log.FatalError(
 """Destination directory
 
 %s
@@ -310,30 +291,23 @@ rdiff-backup like this could mess up what is currently in it.  If you
 want to update or overwrite it, run rdiff-backup with the --force
 option.""" % rpout.path)
 
-	if not rpout.lstat():
-		try: rpout.mkdir()
-		except os.error:
-			Log.FatalError("Unable to create directory %s" % rpout.path)
-	if not datadir.lstat(): datadir.mkdir()
-	inc_base = datadir.append_path("increments")
+	if not Globals.rbdir.lstat(): Globals.rbdir.mkdir()
+	inc_base = Globals.rbdir.append_path("increments")
 	if not inc_base.lstat(): inc_base.mkdir()
 	if Log.verbosity > 0:
-		Log.open_logfile(datadir.append("backup.log"))
+		Log.open_logfile(Globals.rbdir.append("backup.log"))
 	ErrorLog.open(Time.curtimestr, compress = Globals.compression)
-	backup_warn_if_infinite_regress(rpin, rpout)
 
 def backup_warn_if_infinite_regress(rpin, rpout):
 	"""Warn user if destination area contained in source area"""
-	# Just a few heuristics, we don't have to get every case
-	if rpout.conn is not rpin.conn: return
-	if len(rpout.path) <= len(rpin.path)+1: return
-	if rpout.path[:len(rpin.path)+1] != rpin.path + '/': return
-
-	relative_rpout_comps = tuple(rpout.path[len(rpin.path)+1:].split('/'))
-	relative_rpout = rpin.new_index(relative_rpout_comps)
-	if not Globals.select_mirror.Select(relative_rpout): return
-
-	Log(
+	if rpout.conn is rpin.conn: # it's meaningful to compare paths
+		if ((len(rpout.path) > len(rpin.path)+1 and
+			 rpout.path[:len(rpin.path)] == rpin.path and
+			 rpout.path[len(rpin.path)] == '/') or
+			(rpin.path == "." and rpout.path[0] != '/' and
+			 rpout.path[:2] != '..')):
+			# Just a few heuristics, we don't have to get every case
+			if Globals.backup_reader.Globals.select_source.Select(rpout): Log(
 """Warning: The destination directory '%s' may be contained in the
 source directory '%s'.  This could cause an infinite regress.  You
 may need to use the --exclude option.""" % (rpout.path, rpin.path), 2)
@@ -347,6 +321,36 @@ def backup_get_mirrortime():
 	if mirror_rps: return mirror_rps[0].getinctime()
 	else: return None
 
+def backup_set_fs_globals(rpin, rpout):
+	"""Use fs_abilities to set the globals that depend on filesystem"""
+	def update_bool_global(attr, bool):
+		"""If bool is not None, update Globals.attr accordingly"""
+		if Globals.get(attr) is None:
+			SetConnections.UpdateGlobal(attr, bool)
+
+	src_fsa = fs_abilities.FSAbilities('source').init_readonly(rpin)
+	Log(str(src_fsa), 3)
+	dest_fsa = fs_abilities.FSAbilities('destination').init_readwrite(
+		Globals.rbdir, override_chars_to_quote = Globals.chars_to_quote)
+	Log(str(dest_fsa), 3)
+
+	update_bool_global('read_acls', src_fsa.acls)
+	update_bool_global('read_eas', src_fsa.eas)
+	update_bool_global('read_resource_forks', src_fsa.resource_forks)
+
+	SetConnections.UpdateGlobal('preserve_hardlinks', dest_fsa.hardlinks)
+	SetConnections.UpdateGlobal('fsync_directories', dest_fsa.fsync_dirs)
+	SetConnections.UpdateGlobal('change_ownership', dest_fsa.ownership)
+	update_bool_global('write_acls', Globals.read_acls and dest_fsa.acls)
+	update_bool_global('write_eas', Globals.read_eas and dest_fsa.eas)
+	update_bool_global('write_resource_forks',
+					   Globals.read_resource_forks and dest_fsa.resource_forks)
+	update_bool_global('change_dir_inc_perms', dest_fsa.dir_inc_perms)
+	SetConnections.UpdateGlobal('chars_to_quote', dest_fsa.chars_to_quote)
+	if Globals.chars_to_quote:
+		for conn in Globals.connections:
+			conn.FilenameMapping.set_init_quote_vals()
+	
 def backup_touch_curmirror_local(rpin, rpout):
 	"""Make a file like current_mirror.time.data to record time
 
@@ -378,68 +382,81 @@ def backup_remove_curmirror_local():
 	older_inc.delete()
 
 
-def Restore(src_rp, dest_rp = None):
+def Restore(src_rp, dest_rp, restore_as_of = None):
 	"""Main restoring function
 
-	Here src_rp should be an increment file, and if dest_rp is
-	missing it defaults to the base of the increment.
+	Here src_rp should be the source file (either an increment or
+	mirror file), dest_rp should be the target rp to be written.
 
 	"""
-	rpin, rpout = restore_check_paths(src_rp, dest_rp)
-	restore_common(rpin, rpout, rpin.getinctime())
+	if not restore_root_set: assert restore_set_root(src_rp)
+	restore_check_paths(src_rp, dest_rp, restore_as_of)
+	restore_set_fs_globals(dest_rp)
+	src_rp = restore_init_quoting(src_rp)
+	restore_check_backup_dir(restore_root, src_rp, restore_as_of)
+	if restore_as_of:
+		try: time = Time.genstrtotime(restore_timestr)
+		except Time.TimeException, exc: Log.FatalError(str(exc))
+	else: time = src_rp.getinctime()
+	inc_rpath = Globals.rbdir.append_path('increments', restore_index)
+	restore_set_select(restore_root, dest_rp)
+	restore_start_log(src_rp, dest_rp, time)
+	restore.Restore(restore_root.new_index(restore_index),
+					inc_rpath, dest_rp, time)
+	Log("Restore finished", 4)
 
-def RestoreAsOf(rpin, target):
-	"""Secondary syntax for restore operation
+def restore_init_quoting(src_rp):
+	"""Change rpaths into quoted versions of themselves if necessary"""
+	global restore_root
+	if not Globals.chars_to_quote: return src_rp
+	for conn in Globals.connections: conn.FilenameMapping.set_init_quote_vals()
+	restore_root = FilenameMapping.get_quotedrpath(restore_root)
+	SetConnections.UpdateGlobal(
+		'rbdir', FilenameMapping.get_quotedrpath(Globals.rbdir))
+	return FilenameMapping.get_quotedrpath(src_rp)
 
-	rpin - RPath of mirror file to restore (not nec. with correct index)
-	target - RPath of place to put restored file
+def restore_set_fs_globals(target):
+	"""Use fs_abilities to set the globals that depend on filesystem"""
+	def update_bool_global(attr, bool):
+		"""If bool is not None, update Globals.attr accordingly"""
+		if Globals.get(attr) is None: SetConnections.UpdateGlobal(attr, bool)
 
-	"""
-	rpin, rpout = restore_check_paths(rpin, target, 1)
-	try: time = Time.genstrtotime(restore_timestr)
-	except Time.TimeException, exc: Log.FatalError(str(exc))
-	restore_common(rpin, target, time)
+	target_fsa = fs_abilities.FSAbilities('destination').init_readwrite(
+		target, 0)
+	mirror_fsa = fs_abilities.FSAbilities('source').init_readwrite(
+		Globals.rbdir)
 
-def restore_common(rpin, target, time):
-	"""Restore operation common to Restore and RestoreAsOf"""
-	if target.conn.os.getuid() == 0:
-		SetConnections.UpdateGlobal('change_ownership', 1)
-	mirror_root, index = restore_get_root(rpin)
-	restore_check_backup_dir(mirror_root)
-	mirror = mirror_root.new_index(index)
-	inc_rpath = datadir.append_path('increments', index)
-	restore_set_select(mirror_root, target)
-	restore_start_log(rpin, target, time)
-	restore.Restore(mirror, inc_rpath, target, time)
-	Log("Restore ended", 4)
+	update_bool_global('read_acls', target_fsa.acls)
+	update_bool_global('write_acls', target_fsa.acls)
+	update_bool_global('read_eas', target_fsa.eas)
+	update_bool_global('write_eas', target_fsa.eas)
+	update_bool_global('read_resource_forks', target_fsa.resource_forks)
+	update_bool_global('write_resource_forks', target_fsa.resource_forks)
+	SetConnections.UpdateGlobal('preserve_hardlinks', target_fsa.hardlinks)
+	SetConnections.UpdateGlobal('change_ownership', target_fsa.ownership)
+
+	if Globals.chars_to_quote is None: # otherwise already overridden
+		if mirror_fsa.chars_to_quote:
+			SetConnections.UpdateGlobal('chars_to_quote',
+										mirror_fsa.chars_to_quote)
+		else: SetConnections.UpdateGlobal('chars_to_quote', "")
 
 def restore_set_select(mirror_rp, target):
-	"""Set the selection iterator on both side from command line args
+	"""Set the selection iterator on mirror side from command line args
 
-	We must set both sides because restore filtering is different from
-	select filtering.  For instance, if a file is excluded it should
-	not be deleted from the target directory.
-
-	The StringIO stuff is because filelists need to be read and then
-	duplicated, because we need two copies of them now.
+	Here we set the selector on the mirror side, because that is where
+	we will be filtering, but the pathnames are relative to the target
+	directory.
 
 	"""
-	def fp2string(fp):
-		buf = fp.read()
-		assert not fp.close()
-		return buf
-	select_data = map(fp2string, select_files)
 	if select_opts: 
 		mirror_rp.conn.restore.MirrorStruct.set_mirror_select(
-			target, select_opts, *map(cStringIO.StringIO, select_data))
-		target.conn.restore.TargetStruct.set_target_select(
-			target, select_opts, *map(cStringIO.StringIO, select_data))
+			target, select_opts, *select_files)
 
 def restore_start_log(rpin, target, time):
 	"""Open restore log file, log initial message"""
-	try: Log.open_logfile(datadir.append("restore.log"))
-	except (LoggerError, Security.Violation), e:
-		Log("Warning - Unable to open " + str(e), 2)
+	try: Log.open_logfile(Globals.rbdir.append("restore.log"))
+	except LoggerError, e: Log("Warning, " + str(e), 2)
 
 	# Log following message at file verbosity 3, but term verbosity 4
 	log_message = ("Starting restore of %s to %s as it was as of %s." %
@@ -452,34 +469,29 @@ def restore_check_paths(rpin, rpout, restoreasof = None):
 	if not restoreasof:
 		if not rpin.lstat():
 			Log.FatalError("Source file %s does not exist" % rpin.path)
-		if Globals.quoting_enabled:
-			rpin = FilenameMapping.get_quotedrpath(rpin, 1)
-		if not rpin.isincfile():
-			Log.FatalError("""File %s does not look like an increment file.
-
-Try restoring from an increment file (the filenames look like
-"foobar.2001-09-01T04:49:04-07:00.diff").""" % rpin.path)
-
-	if not rpout: rpout = rpath.RPath(Globals.local_connection,
-									  rpin.getincbase_str())
-	if rpout.lstat() and not force:
+	if not force and rpout.lstat() and (not rpout.isdir() or rpout.listdir()):
 		Log.FatalError("Restore target %s already exists, "
 					   "specify --force to overwrite." % rpout.path)
-	return rpin, rpout
 
-def restore_check_backup_dir(rpin):
+def restore_check_backup_dir(mirror_root, src_rp = None, restore_as_of = 1):
 	"""Make sure backup dir root rpin is in consistent state"""
-	result = checkdest_need_check(rpin)
+	if not restore_as_of and not src_rp.isincfile():
+		Log.FatalError("""File %s does not look like an increment file.
+
+Try restoring from an increment file (the filenames look like
+"foobar.2001-09-01T04:49:04-07:00.diff").""" % src_rp.path)
+
+	result = checkdest_need_check(mirror_root)
 	if result is None:
 		Log.FatalError("%s does not appear to be an rdiff-backup directory."
-					   % (rpin.path,))
+					   % (Globals.rbdir.path,))
 	elif result == 1: Log.FatalError(
-		"Previous backup to %s seems to have failed."
-		"Rerun rdiff-backup with --check-destination-dir option to revert"
-		"directory to state before unsuccessful session." % (rpin.path,))
+		"Previous backup to %s seems to have failed.\nRerun rdiff-backup "
+		"rdiff-with --check-destination-dir option to revert directory "
+		"to state before unsuccessful session." % (mirror_root.path,))
 
-def restore_get_root(rpin):
-	"""Return (mirror root, index) and set the data dir
+def restore_set_root(rpin):
+	"""Set data dir, restore_root and index, or return None if fail
 
 	The idea here is to keep backing up on the path until we find
 	a directory that contains "rdiff-backup-data".  That is the
@@ -492,47 +504,46 @@ def restore_get_root(rpin):
 	funny way, using symlinks or somesuch.
 
 	"""
-	global datadir
+	global restore_root, restore_index, restore_root_set
 	if rpin.isincfile(): relpath = rpin.getincbase().path
 	else: relpath = rpin.path
-	if rpin.conn is not Globals.local_connection:
-		# For security checking consistency, don't get absolute path
-		pathcomps = relpath.split('/')
-	else: pathcomps = os.path.join(os.getcwd(), relpath).split("/")
-	if not pathcomps[0]: min_len_pathcomps = 2 # treat abs paths differently
-	else: min_len_pathcomps = 1
+	pathcomps = os.path.join(rpin.conn.os.getcwd(), relpath).split("/")
+	assert len(pathcomps) >= 2 # path should be relative to /
 
 	i = len(pathcomps)
-	while i >= min_len_pathcomps:
+	while i >= 2:
 		parent_dir = rpath.RPath(rpin.conn, "/".join(pathcomps[:i]))
 		if (parent_dir.isdir() and
 			"rdiff-backup-data" in parent_dir.listdir()): break
 		i = i-1
-	else: Log.FatalError("Unable to find rdiff-backup-data directory")
+	else: return None
 
-	if not Globals.quoting_enabled: rootrp = parent_dir
-	else: rootrp = FilenameMapping.get_quotedrpath(parent_dir)
-	Log("Using mirror root directory %s" % rootrp.path, 6)
-
-	datadir = rootrp.append_path("rdiff-backup-data")
-	SetConnections.UpdateGlobal('rbdir', datadir)
-	if not datadir.isdir():
+	restore_root = parent_dir
+	Log("Using mirror root directory %s" % restore_root.path, 6)
+	SetConnections.UpdateGlobal('rbdir',
+								restore_root.append_path("rdiff-backup-data"))
+	if not Globals.rbdir.isdir():
 		Log.FatalError("Unable to read rdiff-backup-data directory %s" %
-					   datadir.path)
+					   Globals.rbdir.path)
 
 	from_datadir = tuple(pathcomps[i:])
 	if not from_datadir or from_datadir[0] != "rdiff-backup-data":
-		return (rootrp, from_datadir) # in mirror, not increments
-	assert from_datadir[1] == "increments"
-	return (rootrp, from_datadir[2:])
+		restore_index = from_datadir # in mirror, not increments
+	else:
+		assert (from_datadir[1] == "increments" or
+				(len(from_datadir) == 2 and
+				 from_datadir[1].startswith('increments'))), from_datadir
+		restore_index = from_datadir[2:]
+	restore_root_set = 1
+	return 1
 
 
 def ListIncrements(rp):
 	"""Print out a summary of the increments and their times"""
-	mirror_root, index = restore_get_root(rp)
-	restore_check_backup_dir(mirror_root)
-	mirror_rp = mirror_root.new_index(index)
-	inc_rpath = Globals.rbdir.append_path('increments', index)
+	assert restore_set_root(rp)
+	restore_check_backup_dir(restore_root)
+	mirror_rp = restore_root.new_index(restore_index)
+	inc_rpath = Globals.rbdir.append_path('increments', restore_index)
 	incs = restore.get_inclist(inc_rpath)
 	mirror_time = restore.MirrorStruct.get_mirror_time()
 	if Globals.parsable_output:
@@ -542,23 +553,22 @@ def ListIncrements(rp):
 
 def ListIncrementSizes(rp):
 	"""Print out a summary of the increments """
-	mirror_root, index = restore_get_root(rp)
-	restore_check_backup_dir(mirror_root)
-	print manage.ListIncrementSizes(mirror_root, index)
+	assert restore_set_root(rp)
+	restore_check_backup_dir(restore_root)
+	print manage.ListIncrementSizes(restore_root, restore_index)
 
 
 def CalculateAverage(rps):
 	"""Print out the average of the given statistics files"""
-	statobjs = map(lambda rp: statistics.StatsObj().read_stats_from_rp(rp),
-				   rps)
-	average_stats = statistics.StatsObj().set_to_average(statobjs)
+	statobjs = map(lambda rp: StatsObj().read_stats_from_rp(rp), rps)
+	average_stats = StatsObj().set_to_average(statobjs)
 	print average_stats.get_stats_logstring(
 		"Average of %d stat files" % len(rps))
 
 
 def RemoveOlderThan(rootrp):
 	"""Remove all increment files older than a certain time"""
-	rot_check_dir(rootrp)
+	rom_check_dir(rootrp)
 	try: time = Time.genstrtotime(remove_older_than_string)
 	except Time.TimeException, exc: Log.FatalError(str(exc))
 	timep = Time.timetopretty(time)
@@ -583,7 +593,7 @@ def RemoveOlderThan(rootrp):
 	else: Log("Deleting increments at times:\n" + inc_pretty_time, 3)
 	manage.delete_earlier_than(Globals.rbdir, time)
 
-def rot_check_dir(rootrp):
+def rom_check_dir(rootrp):
 	"""Check destination dir before RemoveOlderThan"""
 	SetConnections.UpdateGlobal('rbdir',
 								rootrp.append_path("rdiff-backup-data"))
@@ -597,30 +607,27 @@ def ListChangedSince(rp):
 	"""List all the files under rp that have changed since restoretime"""
 	try: rest_time = Time.genstrtotime(restore_timestr)
 	except Time.TimeException, exc: Log.FatalError(str(exc))
-	mirror_root, index = restore_get_root(rp)
-	restore_check_backup_dir(mirror_root)
-	mirror_rp = mirror_root.new_index(index)
-	inc_rp = mirror_rp.append_path("increments", index)
-	for rorp in rp.conn.restore.ListChangedSince(mirror_rp, inc_rp, rest_time):
-		# This is a hack, see restore.ListChangedSince for rational
-		print rorp.index[0]
+	assert restore_set_root(rp)
+	restore_check_backup_dir(restore_root)
+	mirror_rp = restore_root.new_index(restore_index)
+	inc_rp = mirror_rp.append_path("increments", restore_index)
+	restore.ListChangedSince(mirror_rp, inc_rp, rest_time)
 
 
 def ListAtTime(rp):
 	"""List files in archive under rp that are present at restoretime"""
 	try: rest_time = Time.genstrtotime(restore_timestr)
 	except Time.TimeException, exc: Log.FatalError(str(exc))
-	mirror_root, index = restore_get_root(rp)
-	restore_check_backup_dir(mirror_root)
-	mirror_rp = mirror_root.new_index(index)
-	inc_rp = mirror_rp.append_path("increments", index)
-	for rorp in rp.conn.restore.ListAtTime(mirror_rp, inc_rp, rest_time):
-		print rorp.get_indexpath()
+	assert restore_set_root(rp)
+	restore_check_backup_dir(restore_root)
+	mirror_rp = restore_root.new_index(restore_index)
+	inc_rp = mirror_rp.append_path("increments", restore_index)
+	restore.ListAtTime(mirror_rp, inc_rp, rest_time)
 	
 
 def CheckDest(dest_rp):
 	"""Check the destination directory, """
-	if Globals.quoting_enabled:
+	if Globals.chars_to_quote:
 		dest_rp = FilenameMapping.get_quotedrpath(dest_rp)
 	if Globals.rbdir is None:
 		SetConnections.UpdateGlobal('rbdir',
@@ -636,6 +643,9 @@ def CheckDest(dest_rp):
 def checkdest_need_check(dest_rp):
 	"""Return None if no dest dir found, 1 if dest dir needs check, 0 o/w"""
 	if not dest_rp.isdir() or not Globals.rbdir.isdir(): return None
+	if Globals.rbdir.listdir() == ['chars_to_quote']:
+		# This may happen the first backup just after we test for quoting
+		return None
 	curmirroot = Globals.rbdir.append("current_mirror")
 	curmir_incs = restore.get_inclist(curmirroot)
 	if not curmir_incs:
@@ -647,12 +657,6 @@ The rdiff-backup data directory
 exists, but we cannot find a valid current_mirror marker.  You can
 avoid this message by removing this directory; however any data in it
 will be lost.
-
-Probably this error was caused because the first rdiff-backup session
-into a new directory failed.  If this is the case it is safe to delete
-the rdiff_backup_data directory because there is no important
-information in it.
-
 """ % (Globals.rbdir.path,))
 	elif len(curmir_incs) == 1: return 0
 	else:
