@@ -76,8 +76,9 @@ def set_security_level(action, cmdpairs):
 			rdir = tempfile.gettempdir()
 		elif islocal(cp1):
 			sec_level = "read-only"
-			rdir = Main.restore_get_root(rpath.RPath(Globals.local_connection,
-													 getpath(cp1)))[0].path
+			Main.restore_set_root(rpath.RPath(Globals.local_connection,
+											  getpath(cp1)))
+			rdir = Main.restore_root.path
 		else:
 			assert islocal(cp2)
 			sec_level = "all"
@@ -93,10 +94,9 @@ def set_security_level(action, cmdpairs):
 			assert islocal(cp2)
 			sec_level = "all"
 			rdir = getpath(cp2)
-	elif (action == "test-server" or action == "list-increments" or
-		  action == "list-increment-sizes" or action == "list-at-time"
-		  or action == "list-changed-since" or action == "calculate-average"
-		  or action == "remove-older-than"):
+	elif action in ["test-server", "list-increments", 'list-increment-sizes',
+					 "list-at-time", "list-changed-since",
+					 "calculate-average", "remove-older-than"]:
 		sec_level = "minimal"
 		rdir = tempfile.gettempdir()
 	else: assert 0, "Unknown action %s" % action
@@ -112,15 +112,12 @@ def set_allowed_requests(sec_level):
 	allowed_requests = ["VirtualFile.readfromid", "VirtualFile.closebyid",
 						"Globals.get", "Globals.is_not_None",
 						"Globals.get_dict_val",
-						"FilenameMapping.set_init_quote_vals_local",
 						"log.Log.open_logfile_allconn",
 						"log.Log.close_logfile_allconn",
 						"Log.log_to_file",
 						"SetConnections.add_redirected_conn",
 						"RedirectedRun",
-						"sys.stdout.write",
-						"robust.install_signal_handlers",
-						"Hardlink.initialize_dictionaries"]
+						"sys.stdout.write"]
 	if sec_level == "minimal": pass
 	elif sec_level == "read-only" or sec_level == "update-only":
 		allowed_requests.extend(
@@ -129,29 +126,23 @@ def set_allowed_requests(sec_level):
 			 "os.getuid",
 			 "os.listdir",
 			 "Time.setcurtime_local",
+			 "robust.Resume.ResumeCheck",
+			 "backup.SourceStruct.split_initial_dsiter",
+			 "backup.SourceStruct.get_diffs_and_finalize",
 			 "rpath.gzip_open_local_read",
 			 "rpath.open_local_read"])
-		if sec_level == "read-only":
-			allowed_requests.extend(
-				["restore.MirrorStruct.set_mirror_and_rest_times",
-				 "restore.MirrorStruct.initialize_rf_cache",
-				 "restore.MirrorStruct.close_rf_cache",
-				 "restore.MirrorStruct.get_diffs",
-				 "backup.SourceStruct.set_source_select",
-				 "backup.SourceStruct.get_source_select",
-				 "backup.SourceStruct.get_diffs"])
 		if sec_level == "update-only":
 			allowed_requests.extend(
-				["log.Log.open_logfile_local", "log.Log.close_logfile_local",
-				 "log.ErrorLog.open", "log.ErrorLog.isopen",
-				 "log.ErrorLog.close",
+				["Log.open_logfile_local", "Log.close_logfile_local",
+				 "Log.close_logfile_allconn", "Log.log_to_file",
+				 "log.Log.log_to_file",
 				 "robust.SaveState.init_filenames",
 				 "robust.SaveState.touch_last_file",
-				 "backup.DestinationStruct.set_rorp_cache",
-				 "backup.DestinationStruct.get_sigs",				 
-				 "backup.DestinationStruct.patch_and_increment",
+				 "backup.DestinationStruct.get_sigs",
+				 "backup.DestinationStruct.patch_w_datadir_writes",
+				 "backup.DestinationStruct.patch_and_finalize",
+				 "backup.DestinationStruct.patch_increment_and_finalize",
 				 "Main.backup_touch_curmirror_local",
-				 "Main.backup_remove_curmirror_local",
 				 "Globals.ITRB.increment_stat",
 				 "statistics.record_error",
 				 "log.ErrorLog.write_if_open"])
@@ -161,6 +152,7 @@ def set_allowed_requests(sec_level):
 			 "log.Log.setverbosity",
 			 "log.Log.setterm_verbosity",
 			 "Time.setprevtime_local",
+			 "FilenameMapping.set_init_quote_vals_local",
 			 "Globals.postset_regexp_local",
 			 "Globals.set_select",
 			 "backup.SourceStruct.set_session_info",
@@ -187,7 +179,6 @@ def vet_rpath(rpath):
 	"""Require rpath not to step outside retricted directory"""
 	if Globals.restrict_path and rpath.conn is Globals.local_connection:
 		normalized, restrict = rpath.normalize().path, Globals.restrict_path
-		if restrict == "/": return
 		components = normalized.split("/")
 		# 3 cases for restricted dir /usr/foo:  /var, /usr/foobar, /usr/foo/..
 		if (not normalized.startswith(restrict) or
