@@ -1,4 +1,4 @@
-# Copyright 2002 Ben Escoto
+# Copyright 2002, 2003 Ben Escoto
 #
 # This file is part of rdiff-backup.
 #
@@ -22,7 +22,8 @@
 from __future__ import generators
 import errno
 import Globals, metadata, rorpiter, TempFile, Hardlink, robust, increment, \
-	   rpath, static, log, selection, Time, Rdiff, statistics, iterfile
+	   rpath, static, log, selection, Time, Rdiff, statistics, iterfile, \
+	   eas_acls
 
 def Mirror(src_rpath, dest_rpath):
 	"""Turn dest_rpath into a copy of src_rpath"""
@@ -49,7 +50,7 @@ def Mirror_and_increment(src_rpath, dest_rpath, inc_rpath):
 
 class SourceStruct:
 	"""Hold info used on source side when backing up"""
-	_source_select = None # will be set to source Select iterator
+	source_select = None # will be set to source Select iterator
 	def set_source_select(cls, rpath, tuplelist, *filelists):
 		"""Initialize select object using tuplelist
 
@@ -58,7 +59,7 @@ class SourceStruct:
 		connection.  Otherwise we will get an error because a list
 		containing files can't be pickled.
 
-		Also, cls._source_select needs to be cached so get_diffs below
+		Also, cls.source_select needs to be cached so get_diffs below
 		can retrieve the necessary rps.
 
 		"""
@@ -66,16 +67,15 @@ class SourceStruct:
 		sel.ParseArgs(tuplelist, filelists)
 		sel.set_iter()
 		cache_size = Globals.pipeline_max_length * 3 # to and from+leeway
-		cls._source_select = rorpiter.CacheIndexable(sel, cache_size)
-		Globals.set('select_mirror', sel)
+		cls.source_select = rorpiter.CacheIndexable(sel, cache_size)
 
 	def get_source_select(cls):
 		"""Return source select iterator, set by set_source_select"""
-		return cls._source_select
+		return cls.source_select
 
 	def get_diffs(cls, dest_sigiter):
 		"""Return diffs of any files with signature in dest_sigiter"""
-		source_rps = cls._source_select
+		source_rps = cls.source_select
 		error_handler = robust.get_error_handler("ListError")
 		def attach_snapshot(diff_rorp, src_rp):
 			"""Attach file of snapshot to diff_rorp, w/ error checking"""
@@ -107,9 +107,7 @@ class SourceStruct:
 				diff_rorp.flaglinked(dest_sig.get_link_flag())
 			elif dest_sig.isreg() and src_rp.isreg():
 				attach_diff(diff_rorp, src_rp, dest_sig)
-			elif src_rp.isreg():
-				attach_snapshot(diff_rorp, src_rp)
-				dest_sig.close_if_necessary()
+			elif src_rp.isreg(): attach_snapshot(diff_rorp, src_rp)
 			else: diff_rorp.set_attached_filetype('snapshot')
 			yield diff_rorp
 
@@ -125,16 +123,18 @@ class DestinationStruct:
 		destination except rdiff-backup-data directory.
 
 		"""
-		if use_metadata:
-			metadata_iter = metadata.GetMetadata_at_time(Globals.rbdir,
-														 Time.prevtime)
-			if metadata_iter: return metadata_iter
-			log.Log("Warning: Metadata file not found.\n"
-					"Metadata will be read from filesystem.", 2)
+		def get_iter_from_fs():
+			"""Get the combined iterator from the filesystem"""
+			sel = selection.Select(rpath)
+			sel.parse_rbdir_exclude()
+			return sel.set_iter()
 
-		sel = selection.Select(rpath)
-		sel.parse_rbdir_exclude()
-		return sel.set_iter()
+		if use_metadata:
+			rorp_iter = eas_acls.GetCombinedMetadataIter(
+				Globals.rbdir, Time.prevtime,
+				acls = Globals.read_acls, eas = Globals.read_eas)
+			if rorp_iter: return rorp_iter
+		return get_iter_from_fs()
 
 	def set_rorp_cache(cls, baserp, source_iter, for_increment):
 		"""Initialize cls.CCPP, the destination rorp cache
@@ -146,7 +146,7 @@ class DestinationStruct:
 		dest_iter = cls.get_dest_select(baserp, for_increment)
 		collated = rorpiter.Collate2Iters(source_iter, dest_iter)
 		cls.CCPP = CacheCollatedPostProcess(
-			collated, Globals.pipeline_max_length*4, baserp)
+			collated, Globals.pipeline_max_length*4)
 		# pipeline len adds some leeway over just*3 (to and from and back)
 		
 	def get_sigs(cls, dest_base_rpath):
@@ -184,26 +184,15 @@ class DestinationStruct:
 		elif dest_rorp:
 			dest_sig = dest_rorp.getRORPath()
 			if dest_rorp.isreg():
-				sig_fp = cls.get_one_sig_fp(dest_base_rpath.new_index(index))
-				if sig_fp is None: return None
-				dest_sig.setfile(sig_fp)
+				dest_rp = dest_base_rpath.new_index(index)
+				if not dest_rp.isreg():
+					log.ErrorLog.write_if_open("UpdateError", dest_rp,
+						"File changed from regular file before signature")
+					return None
+				dest_sig.setfile(Rdiff.get_signature(dest_rp))
 		else: dest_sig = rpath.RORPath(index)
 		return dest_sig			
 
-	def get_one_sig_fp(cls, dest_rp):
-		"""Return a signature fp of given index, corresponding to reg file"""
-		if not dest_rp.isreg():
-			log.ErrorLog.write_if_open("UpdateError", dest_rp,
-				"File changed from regular file before signature")
-			return None
-		if Globals.process_uid != 0 and not dest_rp.readable():
-			# This branch can happen with root source and non-root
-			# destination.  Permissions are changed permanently, which
-			# should propogate to the diffs
-			assert dest_rp.isowner(), 'no ownership of %s' % (dest_rp.path,)
-			dest_rp.chmod(0400 | dest_rp.getperms())
-		return Rdiff.get_signature(dest_rp)
-				
 	def patch(cls, dest_rpath, source_diffiter, start_index = ()):
 		"""Patch dest_rpath with an rorpiter of diffs"""
 		ITR = rorpiter.IterTreeReducer(PatchITRB, [dest_rpath, cls.CCPP])
@@ -229,9 +218,9 @@ static.MakeClass(DestinationStruct)
 
 
 class CacheCollatedPostProcess:
-	"""Cache a collated iter of (source_rorp, dest_rorp) pairs
+	"""Cache a collated iter of (source_rorp, dest_rp) pairs
 
-	This is necessary for three reasons:
+	This is necessary for two reasons:
 
 	1.  The patch function may need the original source_rorp or
 	    dest_rp information, which is not present in the diff it
@@ -244,11 +233,6 @@ class CacheCollatedPostProcess:
 	    any metadata until we know the file has been procesed
 	    correctly.
 
-	3.  We may lack permissions on certain destination directories.
-	    The permissions of these directories need to be relaxed before
-	    we enter them to computer signatures, and then reset after we
-	    are done patching everything inside them.
-
 	The class caches older source_rorps and dest_rps so the patch
 	function can retrieve them if necessary.  The patch function can
 	also update the processed correctly flag.  When an item falls out
@@ -256,15 +240,15 @@ class CacheCollatedPostProcess:
 	metadata for it.
 
 	"""
-	def __init__(self, collated_iter, cache_size, dest_root_rp):
+	def __init__(self, collated_iter, cache_size):
 		"""Initialize new CCWP."""
 		self.iter = collated_iter # generates (source_rorp, dest_rorp) pairs
 		self.cache_size = cache_size
-		self.dest_root_rp = dest_root_rp
-
 		self.statfileobj = statistics.init_statfileobj()
 		if Globals.file_statistics: statistics.FileStats.init()
-		metadata.OpenMetadata()
+		metadata.MetadataFile.open_file()
+		if Globals.read_eas: eas_acls.ExtendedAttributesFile.open_file()
+		if Globals.read_acls: eas_acls.AccessControlListFile.open_file()
 
 		# the following should map indicies to lists
 		# [source_rorp, dest_rorp, changed_flag, success_flag, increment]
@@ -281,11 +265,6 @@ class CacheCollatedPostProcess:
 		self.cache_dict = {}
 		self.cache_indicies = []
 
-		# Contains a list of pairs (destination_rps, permissions) to
-		# be used to reset the permissions of certain directories
-		# after we're finished with them
-		self.dir_perms_list = []
-
 	def __iter__(self): return self
 
 	def next(self):
@@ -309,21 +288,6 @@ class CacheCollatedPostProcess:
 		"""
 		if source_rorp: Hardlink.add_rorp(source_rorp, source = 1)
 		if dest_rorp: Hardlink.add_rorp(dest_rorp, source = 0)
-		if (dest_rorp and dest_rorp.isdir() and Globals.process_uid != 0
-			and dest_rorp.getperms() % 01000 < 0700):
-			self.unreadable_dir_init(source_rorp, dest_rorp)
-
-	def unreadable_dir_init(self, source_rorp, dest_rorp):
-		"""Initialize an unreadable dir.
-
-		Make it readable, and if necessary, store the old permissions
-		in self.dir_perms_list so the old perms can be restored.
-
-		"""
-		dest_rp = self.dest_root_rp.new_index(dest_rorp.index)
-		dest_rp.chmod(0700 | dest_rorp.getperms())
-		if source_rorp and source_rorp.isdir():
-			self.dir_perms_list.append((dest_rp, source_rorp.getperms()))
 
 	def shorten_cache(self):
 		"""Remove one element from cache, possibly adding it to metadata"""
@@ -338,7 +302,6 @@ class CacheCollatedPostProcess:
 		del self.cache_dict[first_index]
 		self.post_process(old_source_rorp, old_dest_rorp,
 						  changed_flag, success_flag, inc)
-		if self.dir_perms_list: self.reset_dir_perms(first_index)
 
 	def post_process(self, source_rorp, dest_rorp, changed, success, inc):
 		"""Post process source_rorp and dest_rorp.
@@ -359,19 +322,16 @@ class CacheCollatedPostProcess:
 			metadata_rorp = source_rorp
 		else: metadata_rorp = None
 		if metadata_rorp and metadata_rorp.lstat():
-			metadata.WriteMetadata(metadata_rorp)
+			metadata.MetadataFile.write_object(metadata_rorp)
+			if Globals.read_eas and not metadata_rorp.get_ea().empty():
+				eas_acls.ExtendedAttributesFile.write_object(
+					metadata_rorp.get_ea())
+			if Globals.read_acls and not metadata_rorp.get_acl().is_basic():
+				eas_acls.AccessControlListFile.write_object(
+					metadata_rorp.get_acl())
 		if Globals.file_statistics:
 			statistics.FileStats.update(source_rorp, dest_rorp, changed, inc)
 
-	def reset_dir_perms(self, current_index):
-		"""Reset the permissions of directories when we have left them"""
-		dir_rp, perms = self.dir_perms_list[-1]
-		dir_index = dir_rp.index
-		if (current_index > dir_index and
-			current_index[:len(dir_index)] != dir_index):
-			dir_rp.chmod(perms) # out of directory, reset perms now
-			del self.dir_perms_list[-1]
-
 	def in_cache(self, index):
 		"""Return true if given index is cached"""
 		return self.cache_dict.has_key(index)
@@ -410,10 +370,9 @@ class CacheCollatedPostProcess:
 	def close(self):
 		"""Process the remaining elements in the cache"""
 		while self.cache_indicies: self.shorten_cache()
-		while self.dir_perms_list:
-			dir_rp, perms = self.dir_perms_list.pop()
-			dir_rp.chmod(perms)
-		metadata.CloseMetadata()
+		metadata.MetadataFile.close_file()
+		if Globals.read_eas: eas_acls.ExtendedAttributesFile.close_file()
+		if Globals.read_acls: eas_acls.AccessControlListFile.close_file()
 		if Globals.print_statistics: statistics.print_active_stats()
 		if Globals.file_statistics: statistics.FileStats.close()
 		statistics.write_active_statfileobj()
@@ -520,7 +479,6 @@ class PatchITRB(rorpiter.ITRBranch):
 		"""Set self.dir_replacement, which holds data until done with dir
 
 		This is used when base_rp is a dir, and diff_rorp is not.
-		Returns 1 for success or 0 for failure
 
 		"""
 		assert diff_rorp.get_attached_filetype() == 'snapshot'
@@ -530,8 +488,10 @@ class PatchITRB(rorpiter.ITRBranch):
 			# Was an error, so now restore original directory
 			rpath.copy_with_attribs(self.CCPP.get_mirror_rorp(diff_rorp.index),
 									self.dir_replacement)
-			return 0
-		else: return 1
+			success = 0
+		else: success = 1
+		if base_rp.isdir() and Globals.change_permissions: base_rp.chmod(0700)
+		return success
 
 	def prepare_dir(self, diff_rorp, base_rp):
 		"""Prepare base_rp to turn into a directory"""
@@ -543,6 +503,7 @@ class PatchITRB(rorpiter.ITRBranch):
 		else: # maybe no change, so query CCPP before tagging success
 			if self.CCPP.in_cache(diff_rorp.index):
 				self.CCPP.flag_success(diff_rorp.index)
+		if Globals.change_permissions: base_rp.chmod(0700)
 
 	def end_process(self):
 		"""Finish processing directory"""
@@ -621,4 +582,3 @@ class IncrementITRB(PatchITRB):
 				self.CCPP.set_inc(index, inc)
 				self.CCPP.flag_success(index)
 
-
