@@ -79,13 +79,7 @@ def makediff(new, mirror, incpref):
 	if compress: diff = get_inc(incpref, "diff.gz")
 	else:  diff = get_inc(incpref, "diff")
 
-	if Globals.process_uid != 0 and not new.readable():
-		# Check for unreadable files
-		old_new_perms = new.getperms()
-		new.chmod(0400 | old_new_perms)
-		Rdiff.write_delta(new, mirror, diff, compress)
-		new.chmod(old_new_perms)
-	else: Rdiff.write_delta(new, mirror, diff, compress)
+	Rdiff.write_delta(new, mirror, diff, compress)
 	rpath.copy_attribs(mirror, diff)
 	return diff
 
@@ -93,7 +87,10 @@ def makedir(mirrordir, incpref):
 	"""Make file indicating directory mirrordir has changed"""
 	dirsign = get_inc(incpref, "dir")
 	dirsign.touch()
-	if Globals.change_dir_inc_perms: rpath.copy_attribs(mirrordir, dirsign)	
+	if Globals.change_dir_inc_perms:
+		# Below, don't copy acls because directories can have more of
+		# them than ordinary files (they have default acls also).
+		rpath.copy_attribs(mirrordir, dirsign, acls = 0)
 	return dirsign
 
 def get_inc(rp, typestr, time = None):
@@ -111,7 +108,7 @@ def get_inc(rp, typestr, time = None):
 	else:
 		dirname, basename = rp.dirsplit()
 		incrp = rp.__class__(rp.conn, dirname, (addtostr(basename),))
-	assert not incrp.lstat(), incrp
+	assert not incrp.lstat()
 	return incrp
 
 
