@@ -21,7 +21,7 @@
 
 from __future__ import generators
 from log import Log
-import Globals, Time, static, statistics, restore, selection
+import Globals, Time, static, manage, statistics, restore, selection
 
 
 class ManageException(Exception): pass
@@ -200,3 +200,5 @@ def ListIncrementSizes(mirror_root, index):
 		 triple_to_line(triples[0]) + '   (current mirror)']
 	for triple in triples[1:]: l.append(triple_to_line(triple))
 	return '\n'.join(l)
+
+		
