@@ -27,9 +27,9 @@ metadata file (which we assume is intact) and delete the extra
 increments.  For regular files we examine the mirror file and use the
 increment file to get the old data if the mirror file is out of date.
 
-Currently this does not recover hard links.  This may make the
-regressed directory take up more disk space, but hard links can still
-be recovered.
+Currently this does recover hard links.  This make make the regressed
+directory take up more disk space, but hard links can still be
+recovered.
 
 """
 
@@ -65,13 +65,10 @@ def Regress(mirror_rp):
 	assert mirror_rp.conn is inc_rpath.conn is Globals.local_connection
 	set_regress_time()
 	set_restore_times()
-	former_current_mirror_rp = remove_rbdir_increments()
 	ITR = rorpiter.IterTreeReducer(RegressITRB, [])
 	for rf in iterate_meta_rfs(mirror_rp, inc_rpath): ITR(rf.index, rf)
 	ITR.Finish()
-	if former_current_mirror_rp:
-		C.sync() # Sync first, since we are marking dest dir as good now
-		former_current_mirror_rp.delete()
+	remove_rbdir_increments()
 
 def set_regress_time():
 	"""Set global regress_time to previous sucessful backup
@@ -101,39 +98,23 @@ def set_restore_times():
 	restore._rest_time = regress_time
 
 def remove_rbdir_increments():
-	"""Delete the increments in the rdiff-backup-data directory
-
-	Returns the former current mirror rp so we can delete it later.
-	All of the other rp's should be deleted before the actual regress,
-	to clear up disk space the rest of the procedure may need.
-
-	"""
-	former_current_mirror = None
+	"""Delete the increments in the rdiff-backup-data directory"""
+	old_current_mirror = None
 	for filename in Globals.rbdir.listdir():
 		rp = Globals.rbdir.append(filename)
 		if rp.isincfile() and rp.getinctime() == unsuccessful_backup_time:
-			if rp.getincbase_str() == "current_mirror":
-				former_current_mirror = rp
+			if rp.getincbase_str() == "current_mirror": old_current_mirror = rp
 			else:
 				log.Log("Removing rdiff-backup-data increment " + rp.path, 5)
 				rp.delete()
-	return former_current_mirror
+	if old_current_mirror:
+		C.sync() # Sync first, since we are marking dest dir as good now
+		old_current_mirror.delete()
 
 def iterate_raw_rfs(mirror_rp, inc_rp):
-	"""Iterate all RegressFile objects in mirror/inc directory
-
-	Also changes permissions of unreadable files.  We don't have to
-	change them back later because regress will do that for us.
-
-	"""
+	"""Iterate all RegressFile objects in mirror/inc directory"""
 	root_rf = RegressFile(mirror_rp, inc_rp, restore.get_inclist(inc_rp))
 	def helper(rf):
-		mirror_rp = rf.mirror_rp
-		if Globals.process_uid != 0:
-			if mirror_rp.isreg() and not mirror_rp.readable():
-				mirror_rp.chmod(0400 | mirror_rp.getperms())
-			elif mirror_rp.isdir() and not mirror_rp.hasfullperms():
-				mirror_rp.chmod(0700 | mirror_rp.getperms())
 		yield rf
 		if rf.mirror_rp.isdir() or rf.inc_rp.isdir():
 			for sub_rf in rf.yield_sub_rfs():
@@ -143,7 +124,8 @@ def iterate_raw_rfs(mirror_rp, inc_rp):
 
 def yield_metadata():
 	"""Iterate rorps from metadata file, if any are available"""
-	metadata_iter = metadata.GetMetadata_at_time(Globals.rbdir, regress_time)
+	metadata_iter = metadata.MetadataFile.get_objects_at_time(Globals.rbdir,
+															  regress_time)
 	if metadata_iter: return metadata_iter
 	log.Log.FatalError("No metadata for time %s found, cannot regress"
 					   % Time.timetopretty(regress_time))
@@ -251,14 +233,14 @@ class RegressITRB(rorpiter.ITRBranch):
 		if rf.mirror_rp.isreg():
 			tf = TempFile.new(rf.mirror_rp)
 			tf.write_from_fileobj(rf.get_restore_fp())
-			tf.fsync_with_dir() # make sure tf fully written before move
 			rpath.copy_attribs(rf.metadata_rorp, tf)
+			tf.fsync_with_dir() # make sure tf fully written before move
 			rpath.rename(tf, rf.mirror_rp) # move is atomic
 		else:
 			if rf.mirror_rp.lstat(): rf.mirror_rp.delete()
 			rf.mirror_rp.write_from_fileobj(rf.get_restore_fp())
 			rpath.copy_attribs(rf.metadata_rorp, rf.mirror_rp)
-		rf.mirror_rp.get_parent_rp().fsync() # require move before inc delete
+		rf.mirror_rp.fsync_with_dir() # require move before inc delete
 
 	def start_process(self, index, rf):
 		"""Start processing directory"""
@@ -267,7 +249,8 @@ class RegressITRB(rorpiter.ITRBranch):
 			if not rf.mirror_rp.isdir():
 				if rf.mirror_rp.lstat(): rf.mirror_rp.delete()
 				rf.mirror_rp.mkdir()
-			if not rf.mirror_rp.hasfullperms(): rf.mirror_rp.chmod(0700)
+			if Globals.change_permissions and not rf.mirror_rp.hasfullperms():
+				rf.mirror_rp.chmod(0700)
 		self.rf = rf
 
 	def end_process(self):
